(*Mathematica:: Version 7.0 *)
(*:Context:: "surrogate`" *)
(*:Title:: core estimates*)
(*Requirements:*)
 (*Date   2019 Stuart G. Baker *)  					
  



BeginPackage["sfit`", "sfitmanu`", "sfitsimplot`", "sfitplot`","sfitdata`","sfitsim`","sfitlik`","sfitdatasim`"]  

SFit::usage="SFit[data,x]"


Options[SFit]= {SizeOneArmNewTrial->Automatic,ShowCalc->True}
 

SizeOneArmNewTrial::usage="Sample size of  one amr for New Trial"
ShowCalc::usage="Show calculations"

Clear[SFit]

Begin["Private`"]


SFit[data_,options___Rule]:=
Module[{res,dim,lendim,x,sizenew,plot},
 {sizenew,show}={SizeOneArmNewTrial,ShowCalc}/.{options}/.Options[SFit];
 If[InputCheckAll[sizenew,showcalc,data], SFitCore[data,sizenew,show]];   
Return[Null]]


SFitCore[data_,sizenew_,show_]:=
Module[{xvec,yvec,nvec,wvec,name,mat,colname,plot},
 If[show,
      {xvec,yvec,nvec,wvec,name}=data;
      mat=Transpose@{nvec,xvec,yvec,Sqrt[wvec]	};
      colname={"n", "surr eff", "true eff", "sampling se"};
      Print@TableForm[mat,TableHeadings->{None,colname}]];
  If[sizenew===Automatic, 
       Print["using median size for new trial "];   
       nNEW=Median[nvec]//N,
     (*else*)  
       Print["using new trial size ",sizenew];   
       nnew=sizenew];
    (*Plot*)   
      plot=SFitEstimate[data,nNEW]; 
       PlotExport[plot,"fig"];  
    
  Return[Null]]
  
  

SFitEstimate[{xvec_,yvec_,nvec_,wvec_,name_},nNEW_]:=
 Module[{hvec,k,b0,vest,varb},
       k=Length[xvec];
       b0=Total[xvec yvec]/Total[xvec^2];
       v=Max[Total[(yvec-b0 xvec)^2]/k - Total[wvec]/k,0];
       hvec=(yvec-b0 xvec)^2;
        b=Total[xvec yvec/ hvec] /Total[xvec^2/hvec];
        varb= 1/Total[xvec^2 /hvec];
        xNEW=Median[xvec]//N;
        wNEW=Mean[wvec nvec]/nNEW//N;
       varpredNEW=  xNEW^2 varb + v + wNEW//N;
      xdif=Max[xvec]-Min[xvec];
    (*sample size multiplier*) 
       SSM=varpredNEW/wNEW;
    (*prediction separation score*)   
       PSS= b xdif/ (2 1.96 Sqrt[varpredNEW]);
    (*REPORT*)
        namevec={"xNEW", "nNEW", "b0", "v", "b","varb", "wNEW", "varpredNEW", "sample size multiplier", "prediction separation score"};
      resvec=roundx[#]& /@ {xNEW, nNEW, b0,v,b,varb,wNEW,varpredNEW,SSM,PSS};
    mat=Transpose[{namevec,resvec}];
    Print@TableForm[mat];
    (*PLOT*)   
      sepred=Sqrt[varpredNEW];
      est= b x;
      upp=est+ 1.96 sepred;
       low=est-1.96 sepred;
       xmin=Min[xvec];
       xmax=Max[xvec];
        pair=Transpose[{xvec,yvec}];
       plotlines=Flatten@GenLines[{xvec,yvec,nvec,wvec}]; 
        color=Blue;   
        dash=Dashing[{.03,.03}];
        xdif=xmax-xmin;
	xmin0=xmin-.05 xdif;
	xmax0=xmax+.05 xdif;
	ydif=ymax-ymin;
	 plot1=Plot[est,{x,xmin,xmax},PlotStyle->color];
         plot2=Plot[low,{x,xmin,xmax},PlotRange->All,PlotStyle->{color,dash}];
        plot3=Plot[upp,{x,xmin,xmax},PlotRange->All,PlotStyle->{color,dash}];
        plot=Show[plotlines,plot1,plot2,plot3,  Frame->True,
	    FrameLabel->{"Effect of treatment on the surrogate endpoint", "Effect of treatment on the true endpoint", 
	    "Surrogate endpoint meta-analysis",None},
	    BaseStyle->{Bold, Medium},
     	  PlotRange->All,AspectRatio->1];
Return[plot]]


roundx[x_]:=PaddedForm[x,3]



InputCheckAll[sizenew_,showcalc_,data_]:=
Module[{check1,check2,check3, check,len,xvec,yvec,wvec,nvec,name,lenx,leny,lenn,lenw},
 If[sizenew===Automatic || (NumberQ[sizenew] && sizenew>0), check1 =True, check1=False];
 If[show==True || show==False, check2=True, check2==False];
 If[check1 ==False, Print["invalid size"]];
 If[check2 ==False, Print["show calc"]];
  dim=Dimensions[data];
  lendim=Length[dim];
   If[lendim ==0	 || lendim>1, check3 =False];
   If[lendim==1, check3=InputCheck[data]];
   If[lendim==2, check3vec=InputCheck[#]& /@ data; 
          check3=Apply[And,check3vec]];
If[check3==False, Print["Invalid data input"]];
check=check1 && check2 && check3;
Return[check]]

InputCheck[data_]:=
Module[{type,xvec,yvec,nvec,wvec,name, len,len0,lenx,lenw},
len=Length[data];
If[len=!=5,go=False; Print["invalid input length"]];
If[len==5,
{xvec,yvec,nvec,wvec,name}=data;
{lenx,leny,lenn, lenw, len1}=Length[#]& /@ data;
If[lenx> 1 && lenx==leny && lenx==lenn && lenx==lenw, go=True,go=False; Print["invalid input"]]];
Return[go]]


GenLines[{xvec_,yvec_,nvec_,wvec_}]:=
Module[{dvec,tri,set},
tri=Transpose[{xvec,yvec,nvec,wvec}];
set=GenLineOne[#]& /@tri;
Return[set]]


GenLineOne[{x_,y_,n_,w_}]:=
Module[{se,line,d,botline,topline,res},
se=Sqrt[w];
line=Graphics@ Line[{{x,y-1.96 se},{x, y+1.96 se}}];
d=.02;
topline=Graphics@ Line[{{x-d/2,y+1.96 se},{x+d/2, y+1.96 se}}];
botline=Graphics@ Line[{{x-d/2,y-1.96 se},{x+d/2, y-1.96 se}}];
point=Graphics@{PointSize[d], Point[{x, y}]};
res ={line,topline,botline,point};
res={line,point};
Return[res]]

PlotExport[plot_,figname_]:=
	  Module[{plotname},
	 plotname=StringJoin["sfit",figname,".jpeg"];    
	 Export[plotname,plot]; 
	 Print["    exporting ",plotname];
	 Print[plot];
 Return[Null]]


End[] 
EndPackage[]

Print["SFit[data]" ];
Print["SFitManu[ ] to reproduce manuscript results"];
